#!/bin/usr/binpython
#
# Python 3 specific implementations.
# Provided to separate non-portable syntax across python 2 and 3.
#   - Cameron Simpson <cs@cskk.id.au> 12nov2015
#

from builtins import bytes

DISTINFO = {
    'description': "python 3 specific support for cs.py3 module",
    'keywords': ["python2"],
    'classifiers': [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        ],
    'install_requires': [],
}

def raise3(exc_type, exc_value, exc_traceback):
  raise exc_type(exc_value).with_traceback(exc_traceback)

exec_code = exec

# file interface returning bytes from binary file
BytesFile = lambda fp: fp

joinbytes = b''.join



#!/usr/bin/python -tt
#
# Python3 helpers to aid code sharing between python2 and python3.
#       - Cameron Simpson <cs@cskk.id.au> 28jun2012
#

r'''
Aids for code sharing between python2 and python3.

Presents various names in python 3 flavour for common use in python 2 and python 3.
'''

try:
  from configparser import ConfigParser
except ImportError:
  from ConfigParser import SafeConfigParser as ConfigParser
import os
try:
  from queue import Queue, PriorityQueue, Full as Queue_Full, Empty as Queue_Empty
except ImportError:
  from Queue import Queue, Full as Queue_Full, Empty as Queue_Empty
  try:
    from Queue import PriorityQueue
  except ImportError:
    pass
import sys
try:
  from types import StringTypes
except ImportError:
  StringTypes = (str,)

DISTINFO = {
    'keywords': ["python2", "python3"],
    'classifiers': [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        ],
    'install_requires': [],
}

if sys.hexversion >= 0x03000000:

  unicode = str
  def ustr(s, e='utf-8', errors='strict'):
    ''' Upgrade string to unicode: no-op for python 3.
    '''
    return s
  def iteritems(o):
    return o.items()
  def iterkeys(o):
    return o.keys()
  def itervalues(o):
    return o.values()
  from builtins import sorted, filter, bytes, input
  from itertools import filterfalse
  from struct import pack, unpack
  from ._for3 import raise3, exec_code, BytesFile, joinbytes

else:

  globals()['unicode'] = unicode
  bytesjoin = ''.join
  def iteritems(o):
    return o.iteritems()
  def iterkeys(o):
    return o.iterkeys()
  def itervalues(o):
    return o.itervalues()
  input = raw_input
  _sorted = sorted
  def sorted(iterable, key=None, reverse=False):
    return _sorted(iterable, None, key, reverse)
  input = raw_input
  from itertools import ifilter as filter, ifilterfalse as filterfalse
  from ._for2 import raise3, exec_code, ustr, \
                        bytes, BytesFile, joinbytes, \
                        pack, unpack

try:
  from struct import iter_unpack
except ImportError:
  from struct import calcsize
  def iter_unpack(fmt, buffer):
    chunk_size = calcsize(fmt)
    if chunk_size < 1:
      raise ValueError("struct.calcsize(%r) gave %d, expected >= 1" % (fmt, chunk_size))
    offset = 0
    while offset < len(buffer):
      yield unpack(fmt, buffer[offset:offset+chunk_size])
      offset += chunk_size

# fill in missing pread with weak workalike
try:
  pread = os.pread
except AttributeError:
  # implement our own pread
  # NB: not thread safe!
  from os import SEEK_CUR, SEEK_SET
  def pread(fd, size, offset):
    offset0 = os.lseek(fd, 0, SEEK_CUR)
    os.lseek(fd, offset, SEEK_SET)
    chunks = []
    while size > 0:
      data = os.read(fd, size)
      if not data:
        break
      chunks.append(data)
      size -= len(data)
    os.lseek(fd, offset0, SEEK_SET)
    data = b''.join(chunks)
    return data

if __name__ == '__main__':
  import cs.py3_tests
  cs.py3_tests.selftest(sys.argv)



#!/usr/bin/python
#
# Python 2 specific implementations.
# Provided to separate non-portable syntax across python 2 and 3.
#   - Cameron Simpson <cs@cskk.id.au> 12nov2015
# 

DISTINFO = {
    'description': "python 2 specific support for cs.py3 module",
    'keywords': ["python2"],
    'classifiers': [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        ],
    'install_requires': [],
}

def raise3(exc_type, exc_value, exc_traceback):
  exec("raise exc_type, exc_value, exc_traceback")

def exec_code(code, *a):
  if not a:
    exec(code)
  else:
    gs = a.pop(0)
    if not a:
      exec("exec code in gs")
    else:
      ls = a.pop(0)
      if not a:
        exec("exec code in gs, ls")
      else:
        raise ValueError("exec_code: extra arguments after locals: %r" % (a,))

def ustr(s, e='utf-8', errors='strict'):
  ''' Upgrade str to unicode, if it is a str. Leave other types alone.
  '''
  if isinstance(s, str):
    try:
      s = s.decode(e, errors)
    except UnicodeDecodeError as ude:
      from cs.logutils import warning
      warning("cs.py3.ustr(): %s: s = %s %r", ude, type(s), s)
      s = s.decode(e, 'replace')
  return s

class bytes(object):
  ''' Trite bytes implementation.
  '''
  def __init__(self, arg):
    if isinstance(arg, str):
      # accept a str as is
      self.__s = arg
    else:
      # not a str: should be interable of ints or length
      try:
        bytevals = [ b for b in arg ]
      except TypeError:
        bytevals = [ 0 for i in range(arg) ]
      bvals = list(bytevals)
      self.__s = ''.join( chr(b) for b in bytevals )
      if len(self.__s) != len(bvals):
        raise TypeError("bvals=%r, __s=%r", bvals, self.__s)
    if not isinstance(self.__s, str):
      raise TypeError("__s is not a str!")
  def __repr__(self):
    return 'b' + repr(self.__s)
  def __iter__(self):
    for _ in self.__s:
      yield ord(_)
  def __getitem__(self, index):
    return ord(self.__s[index])
  def __getslice__(self, i, j):
      return bytes( ord(_) for _ in self.__s[i:j] )
  def __contains__(self, b):
    return chr(b) in self.__s
  def __eq__(self, other):
    if type(other) is type(self):
      return self.__s == other.__s
    if len(other) != len(self):
      return False
    if isinstance(other, str):
      other = bytes(other)
    for i, b in enumerate(self):
      if b != other[i]:
        return False
    return True
  def __len__(self):
    return len(self.__s)
  def __add__(self, other):
    return bytes( list(self) + list(bytes(other)) )
  def as_str(self):
    ''' Back convert to a str, only meaningful for Python 2.
    '''
    return self.__s
    ##return ''.join( chr(_) for _ in self )
  def as_buffer(self):
    ''' For python 2, support buffer protocol.
    '''
    return self.__s
  @staticmethod
  def join(bss):
    return bytes(''.join(bss))
  def decode(self, encoding='ascii', errors='strict'):
    return self.__s.decode(encoding, errors)
  def find(self, sub, *start_end):
    start_end = list(start_end)
    if start_end:
      start = start_end.pop(0)
    else:
      start = 0
    if start_end:
      end = start_end.pop(0)
    else:
      end = len(self)
    if start_end:
      raise TypeError('find() takes 2 to 4 arguments: extra arguments: %r'
                      % (start_end,))
    return self.__s.find(sub, start, end)

class BytesFile(object):
  ''' Wrapper class for a file opened in binary mode which uses bytes in its methods instead of str.
  '''

  def __init__(self, fp):
    self.fp = fp

  def read(self, *a, **kw):
    s = self.fp.read(*a, **kw)
    if not isinstance(s, str):
      raise TypeError("s=%s %r from %s.read", type(s), s, type(self.fp))
    return bytes(s)
    ##return bytes(self.fp.read(*a, **kw))

  def write(self, bs, *a, **kw):
    if isinstance(bs, str):
      s = bs
    else:
      s = bs.as_str()
    return self.fp.write(s, *a, **kw)

  def seek(self, *a, **kw):
    return self.fp.seek(*a, **kw)

  def flush(self):
    return self.fp.flush()

  def close(self):
    return self.fp.close()

joinbytes = bytes.join

from struct import pack as _pack, unpack as _unpack

def pack(fmt, *values):
  return bytes(_pack(fmt, *values))

def unpack(fmt, bs):
  ##from cs.logutils import X
  ##X("py3_for2.unpack: fmt=%r, bs=%r", fmt, bs)
  if isinstance(bs, bytes):
    bs = bs._bytes__s
    ##X("py3_for2.unpack: bs => %r", bs)
  return _unpack(fmt, bs)



#!/usr/bin/env python
from setuptools import setup
setup(
  name = 'cs.py3',
  description = 'Aids for code sharing between python2 and python3.',
  author = 'Cameron Simpson',
  author_email = 'cs@cskk.id.au',
  version = '20181108',
  url = 'https://bitbucket.org/cameron_simpson/css/commits/all',
  classifiers = ['Programming Language :: Python', 'Programming Language :: Python :: 2', 'Programming Language :: Python :: 3', 'Development Status :: 4 - Beta', 'Intended Audience :: Developers', 'Operating System :: OS Independent', 'Topic :: Software Development :: Libraries :: Python Modules', 'License :: OSI Approved :: GNU General Public License v3 (GPLv3)'],
  install_requires = [],
  keywords = ['python2', 'python3'],
  long_description = "Aids for code sharing between python2 and python3.\n\nPresents various names in python 3 flavour for common use in python 2 and python 3.\n\n## Function `ustr(s, e='utf-8', errors='strict')`\n\nUpgrade string to unicode: no-op for python 3.",
  long_description_content_type = 'text/markdown',
  package_dir = {'': 'lib/python'},
  packages = ['cs.py3'],
)



#!/usr/bin/python
#
# Self tests for cs.py3.
#       - Cameron Simpson <cs@cskk.id.au>
#

import sys
import unittest
from . import bytes

class TestBytes(unittest.TestCase):

  def setUp(self):
    pass

  def tearDown(self):
    pass

  def test00bytes(self):
    b = bytes(10)
    self.assertIsInstance(b, bytes)
    self.assertEqual(len(b), 10)
    self.assertEqual(b[0], 0)
    self.assertEqual(b[9], 0)
    self.assertTrue( all( b[i] == 0 for i in range(len(b)) ) )
    self.assertEqual(b[3:3], bytes( () ))
    self.assertEqual(b[3:4], bytes( (0,) ))
    self.assertEqual(b[3:5], bytes( (0, 0) ))

def selftest(argv):
  unittest.main(__name__, None, argv)

if __name__ == '__main__':
  selftest(sys.argv)
